@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -27,27 +27,11 @@
 
 #include <config.h>
 
-#include <unistd.h>
 #include <stdio.h>
-#include <assert.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
 #include <string.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <errno.h>
 #include <stdlib.h>
-#include <time.h>
-#include <sys/time.h>
 #include <sys/types.h>
 
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <base64.h>
-#else
-#  include <libavutil/base64.h>
-#endif
-
 #define LOG_MODULE "rtsp"
 #define LOG_VERBOSE
 /*
@@ -75,7 +59,6 @@ struct rtsp_s {
   char         *user_agent;
 
   char         *server;
-  unsigned int  server_state;
   uint32_t      server_caps;
 
   unsigned int  cseq;
@@ -132,20 +115,25 @@ static char *rtsp_get(rtsp_t *s) {
  * rtsp_put puts a line on stream
  */
 
-static void rtsp_put(rtsp_t *s, const char *string) {
+static int rtsp_put(rtsp_t *s, const char *string) {
 
+  int ret;
   size_t len=strlen(string);
   char *buf = malloc(sizeof(char)*len+2);
 
   lprintf(">> '%s'", string);
 
+  if (!buf)
+    return -1;
   memcpy(buf,string,len);
   buf[len]=0x0d;
   buf[len+1]=0x0a;
 
-  _x_io_tcp_write(s->stream, s->s, buf, len+2);
+  ret = _x_io_tcp_write (s->stream, s->s, buf, len + 2);
 
+  free(buf);
   lprintf("done.\n");
+  return ret;
 }
 
 /*
@@ -186,17 +174,19 @@ static void rtsp_send_request(rtsp_t *s, const char *type, const char *what) {
   char *buf;
 
   buf = _x_asprintf("%s %s %s",type, what, rtsp_protocol_version);
-  rtsp_put(s,buf);
-
-  if (s->auth) {
-    rtsp_put(s, s->auth);
+  if (buf) {
+    rtsp_put (s, buf);
+    free (buf);
+    if (s->auth)
+      rtsp_put (s, s->auth);
+    if (payload) {
+      while (*payload) {
+        rtsp_put (s, *payload);
+        payload++;
+      }
+    }
   }
 
-  if (payload)
-    while (*payload) {
-      rtsp_put(s,*payload);
-      payload++;
-    }
   rtsp_put(s,"");
   rtsp_unschedule_all(s);
 }
@@ -216,6 +206,7 @@ static void rtsp_schedule_standard(rtsp_t *s) {
     char *buf;
     buf = _x_asprintf("Session: %s", s->session);
     rtsp_schedule_field(s, buf);
+    free(buf);
   }
 }
 /*
@@ -262,6 +253,7 @@ static int rtsp_get_answers(rtsp_t *s) {
         if (strcmp(tmp, s->session)) {
           xprintf(s->stream->xine, XINE_VERBOSITY_DEBUG,
 		  "rtsp: warning: setting NEW session: %s\n", tmp);
+          free(s->session);
           s->session=strdup(tmp);
         }
       } else
@@ -273,7 +265,7 @@ static int rtsp_get_answers(rtsp_t *s) {
     }
     *answer_ptr=answer;
     answer_ptr++;
-  } while ((strlen(answer)!=0) && (++ans_count < MAX_FIELDS));
+  } while ((strlen(answer)!=0) && (++ans_count < MAX_FIELDS-1));
 
   s->cseq++;
 
@@ -441,19 +433,19 @@ int rtsp_read_data(rtsp_t *s, void *buffer_gen, unsigned int size) {
 static void rtsp_basicauth (const char *user, const char *password, char** dest) {
   const size_t totlen = strlen(user) + (password ? strlen(password) : 0) + 1;
   const size_t enclen = ((totlen + 2) * 4 ) / 3 + 12;
-  char         tmp[totlen + 1];
+  char         tmp[totlen + 4];
 
   snprintf(tmp, totlen + 1, "%s:%s", user, password ? : "");
 
   *dest = malloc(enclen);
-  av_base64_encode(*dest, enclen, tmp, totlen);
+  xine_base64_encode ((unsigned char *)tmp, *dest, totlen);
 }
 
 rtsp_t *rtsp_connect(xine_stream_t *stream, const char *mrl, const char *user_agent) {
 
-  rtsp_t *s = malloc(sizeof(rtsp_t));
-  char *mrl_ptr=strdup(mrl);
-  char *slash, *colon, *amp;
+  rtsp_t *s = calloc(1, sizeof(rtsp_t));
+  const char *mrl_ptr = mrl;
+  const char *slash, *colon, *amp;
   int hostend, i;
   size_t pathbegin;
 
@@ -479,7 +471,7 @@ rtsp_t *rtsp_connect(xine_stream_t *stream, const char *mrl, const char *user_ag
   s->mrl=strdup(mrl);
 
   s->server=NULL;
-  s->server_state=0;
+  s->s = -1;
   s->server_caps=0;
 
   s->cseq=0;
@@ -548,8 +540,6 @@ rtsp_t *rtsp_connect(xine_stream_t *stream, const char *mrl, const char *user_ag
     return NULL;
   }
 
-  s->server_state=RTSP_CONNECTED;
-
   /* now lets send an options request. */
   rtsp_schedule_field(s, "CSeq: 1");
   rtsp_schedule_field(s, s->user_agent);
@@ -572,12 +562,13 @@ rtsp_t *rtsp_connect(xine_stream_t *stream, const char *mrl, const char *user_ag
 
 void rtsp_close(rtsp_t *s) {
 
-  if (s->server_state) close(s->s); /* TODO: send a TEAROFF */
-  if (s->path) free(s->path);
-  if (s->host) free(s->host);
-  if (s->mrl) free(s->mrl);
-  if (s->session) free(s->session);
-  if (s->user_agent) free(s->user_agent);
+  if (s->s >= 0)
+    _x_io_tcp_close(s->stream, s->s); /* TODO: send a TEAROFF */
+  free(s->path);
+  free(s->host);
+  free(s->mrl);
+  free(s->session);
+  free(s->user_agent);
   free(s->auth);
   rtsp_free_answers(s);
   rtsp_unschedule_all(s);
@@ -589,12 +580,11 @@ void rtsp_close(rtsp_t *s) {
  * after the first matched tag. returns NULL if no match found.
  */
 
-char *rtsp_search_answers(rtsp_t *s, const char *tag) {
+const char *rtsp_search_answers(rtsp_t *s, const char *tag) {
 
   char **answer;
   char *ptr;
 
-  if (!s->answers) return NULL;
   answer=s->answers;
 
   while (*answer) {
@@ -618,8 +608,7 @@ char *rtsp_search_answers(rtsp_t *s, const char *tag) {
 
 void rtsp_set_session(rtsp_t *s, const char *id) {
 
-  if (s->session) free(s->session);
-
+  free(s->session);
   s->session=strdup(id);
 
 }
@@ -684,7 +673,6 @@ void rtsp_unschedule_all(rtsp_t *s) {
 
   char **ptr;
 
-  if (!s->scheduled) return;
   ptr=s->scheduled;
 
   while (*ptr) {
@@ -701,7 +689,6 @@ void rtsp_free_answers(rtsp_t *s) {
 
   char **answer;
 
-  if (!s->answers) return;
   answer=s->answers;
 
   while (*answer) {
