@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -27,23 +27,9 @@
 
 #include <config.h>
 
-#include <unistd.h>
 #include <stdio.h>
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
 #include <string.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <errno.h>
 #include <stdlib.h>
-#include <sys/time.h>
 #include <sys/types.h>
 
 #define LOG_MODULE "pnm"
@@ -380,6 +366,7 @@ static int pnm_send_request(pnm_t *p, uint32_t bandwidth) {
   int c=PNM_HEADER_SIZE;
   char fixme[]={0,1};
 
+  (void)bandwidth;
   memcpy(p->buffer,pnm_header,PNM_HEADER_SIZE);
   c+=pnm_write_chunk(PNA_CLIENT_CHALLANGE,strlen(pnm_challenge),
           pnm_challenge,&p->buffer[c]);
@@ -448,7 +435,7 @@ static int pnm_send_response(pnm_t *p, const char *response) {
 
   memcpy(&p->buffer[3], response, size);
 
-  if (_x_io_tcp_write(p->stream, p->s, p->buffer, size+3) != size+3)
+  if ((size_t)_x_io_tcp_write (p->stream, p->s, p->buffer, size + 3) != size + 3)
     return 0;
   return 1;
 }
@@ -640,6 +627,11 @@ static int pnm_get_stream_chunk(pnm_t *p) {
   {
     int size=_X_BE_16 (&p->buffer[1]);
 
+    if (size < 5)
+      return 0;
+    if (size + 3 >= BUF_SIZE)
+      return 0;
+
     if (_x_io_tcp_read (p->stream, p->s, &p->buffer[8], size-5) != size-5)
       return 0;
     p->buffer[size+3]=0;
@@ -752,6 +744,7 @@ pnm_t *pnm_connect(xine_stream_t *stream, const char *mrl) {
   p->port=7070;
   p->url=strdup(mrl);
   p->packet=0;
+  p->s = -1;
 
   slash=strchr(mrl_ptr,'/');
   colon=strchr(mrl_ptr,':');
@@ -780,11 +773,7 @@ pnm_t *pnm_connect(xine_stream_t *stream, const char *mrl) {
 
   if (fd == -1) {
     xprintf (p->stream->xine, XINE_VERBOSITY_LOG, _("input_pnm: failed to connect '%s'\n"), p->host);
-    free(p->path);
-    free(p->host);
-    free(p->url);
-    free(p);
-    return NULL;
+    goto fail;
   }
   p->s=fd;
 
@@ -810,10 +799,7 @@ pnm_t *pnm_connect(xine_stream_t *stream, const char *mrl) {
 
  fail:
   xprintf (p->stream->xine, XINE_VERBOSITY_LOG, _("input_pnm: failed to set up stream\n"));
-  free(p->path);
-  free(p->host);
-  free(p->url);
-  free(p);
+  pnm_close(p);
   return NULL;
 }
 
@@ -861,7 +847,8 @@ int pnm_peek_header (pnm_t *this, char *data, int maxsize) {
 
 void pnm_close(pnm_t *p) {
 
-  if (p->s >= 0) close(p->s);
+  if (p->s >= 0)
+    _x_io_tcp_close(p->stream, p->s);
   free(p->path);
   free(p->host);
   free(p->url);
