@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -22,6 +22,10 @@
  * plugins for VDR
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <xine/xine_internal.h>
 #include <xine/post.h>
 #include "combined_vdr.h"
@@ -39,6 +43,6 @@ const plugin_info_t xine_plugin_info[] EXPORTED =
   { PLUGIN_POST ,  10, "vdr"      , XINE_VERSION_CODE, &vdr_video_special_info, &vdr_video_init_plugin },
   { PLUGIN_POST ,  10, "vdr_video", XINE_VERSION_CODE, &vdr_video_special_info, &vdr_video_init_plugin },
   { PLUGIN_POST ,  10, "vdr_audio", XINE_VERSION_CODE, &vdr_audio_special_info, &vdr_audio_init_plugin },
-  { PLUGIN_NONE ,   0, ""         , 0                , NULL                   , NULL }
+  { PLUGIN_NONE ,   0, NULL       , 0                , NULL                   , NULL }
 };
 
